% This code will perform DDM analysis on a TIFF stack by calling FFTImDiff
% and AzimuthalAverage.m. These subroutines perform the bulk of the
% calculation and should be placed inside a loop that steps forward in
% time.
clear all
close all
clc

%% User defined variables
%These are the parameters that need to be entered
folder = 'C:\Users\rdsku\OneDrive\Professional\Codes\DDM\';
file_name = 'Tutorial_Small_190402_800nm_Water_30fps_20x.tif';
file = strcat(folder,file_name);
pixel_size = 0.358;              %Size of pixel in um
num_q = 50;                    %Number of q's
video = 0;                      %1 if we want a video, 0 if not
num_average = 0;                %1 if we want to specify total averaged
frame_rate = 30;
bleach_correction = 0;          %1 if we want to correct for photobleaching          

SingleExpFit = 1;               %1 if we want to fdit to a single exponential
SingleInitGuess = [3e6,1e7,1];
Singlelb = [1e6,1e4,0.001];
Singleub = [2e7,2e8,100];
% SingleGlobalDefVec = [0, 0, 0];

StretchExpFit = 0;               %1 if we want to fit to a single exponential
StretchDynamFit = 0;                %1 if we want to dynamically change initial guesses on upper and lower bounds
StretchFixBaseline = 0;          %1 if we want to set baseline to value at shortest time at highest q
StretchInitGuess = [3e5,3e6,0.1,1];
Stretchlb = [1e5,1e5,1e-5,0];
Stretchub = [1e7,1e9,100,2];

DoubleExpFit = 0;               %1 if we want to fit to a single exponential
DEInitGuess = [4e8,1e9,0.5,0.1,2];
DElb = [3e8,0,0,1e-3,0.5];
DEub = [3e9,6e9,1,1,10];

%Only necessary for video
min_logI = 19;                  %Minimum value of log(FFT) that will be accepted
max_logI = 23;                  %Maximum ''
color_map = jet(8);             %Setting color map for output video

%% Automatic variables

%Input into subroutines requires output of VideoReader on the avi
movie = imfinfo(file);
video_name = strcat('FFT_',file_name);
if video
    if exist(video_name,'file')
        error('The video file already exists. Please delete it')
    end
end

xpix = movie.Width;
ypix = movie.Height;
% qmin = round(2*4*pi./(xpix.*pixel_size),2);   %Minimum q (in um^-1)
% qmax = round(2*pi./(3*pixel_size),1);             %Maximimum q (in um^-1)
% q = logspace(log10(qmin),log10(qmax),num_q);              %Creates a vector of q values that we will use to calculate azimuthal averages (in um^-1)

min_frames = 1;               %Minimum number of averaged frames required
%%
if num_average
    prompt = 'How many frames do you want averaged? \n';
    num_average = input(prompt);
    frames = numel(movie) - max(min_frames,num_average);  %Number of time steps to perform
else
    clear num_average
    frames = numel(movie) - min_frames;  %Number of time steps to perform

end
             
%Determines if you calclulate 'all', 'lin', 'log', or 'custom' times. I do
%not recommend using log because you will be computing a lot of the low
%time steps multiple times. A lot of work without any gain.
prompt = 'What kind of time steps do you want to use? All, custom, lin, log \n';
steps = input(prompt,'s');   

if strcmpi(steps,'custom')
    prompt = 'What time vector would you like to use? \n';
    t_frame = input(prompt); %Set steps_custom = vector of choice
elseif strcmpi(steps,'all')
    t_frame = round(1:frames);
elseif strcmpi(steps,'lin')
    prompt = 'What step size would you like to use? \n';
    step_size = input(prompt);
    t_frame = round(1:step_size:frames);
elseif strcmpi(steps,'log')
    prompt = 'How many times do you want to calculate in log spacing? \n';
    log_num = input(prompt);
    t_frame = unique(round(logspace(log10(1),log10(frames),log_num)));
else
    error('InvalidTimeVector','Did not input a valid type of time vector');
end


%% We will now perform the DDM analysis
fq = zeros(length(t_frame),num_q);        %Creates a vector that will hold q in the columns and lag time in the rows
err_fq = zeros(size(fq));

for i = 1:length(t_frame)     %Runs analysis over the range of t values
    tic;
    lag = t_frame(i);         %Samples the ith entry of t since t is specified above
    
    %We first want to check if we have a specified number of frames to average
    if exist('num_average','var')
        fprintf('Averaging %d frames per FFT \n', num_average)
        FMat = FFTImDiff(movie,lag,num_average);
%         [fq(i,:),err_fq(i,:)] = AzimuthalAverage(FMat,xpix,ypix,pixel_size,q,360);
%         [fq(i,:),err_fq(i,:),q_radavg] = radialavg(FMat,num_q,0,0);
        [fq(i,:),err_fq(i,:),q] = AzimuthalAverage_Updated190911RPS(FMat,xpix,ypix,pixel_size,num_q);
        to = toc;
        if video
            FourierTIF(FMat, video_name, min_logI, max_logI, color_map)
        end
        fprintf('Lag time = %d out of %d \n', i, numel(t_frame));
        fprintf('Iteration time = %d \n', round(to));
        fprintf('Estimated Time Remaining = %d \n', round(to*(length(t_frame)-i)));
    else
        fprintf('Averaging %d frames per FFT \n', (numel(movie)-lag));
        FMat = FFTImDiff(movie,lag);
%         [fq(i,:),err_fq(i,:)] = AzimuthalAverage(FMat,xpix,ypix,pixel_size,q,360);
        [fq(i,:),err_fq(i,:),q] = AzimuthalAverage_Updated190911RPS(FMat,xpix,ypix,pixel_size,num_q);
        if video
            FourierTIF(FMat, video_name, min_logI, max_logI, color_map)
        end
        to = toc;
        fprintf('Lag time = %d out of %d \n', i, numel(t_frame));
        fprintf('Iteration time = %d s \n', round(to));
        fprintf('Estimated Time Remaining = %d s  \n', round(to*(length(t_frame)-i)));
    end
    
end

%%
n_ex = isnan(fq(1,:)) + isinf(1./fq(1,:)) ;
n_tot = numel(n_ex)-sum(n_ex);

fq_w_Err = NaN(size(fq,1),2*n_tot);
q_new = zeros(1,n_tot);

n = 1;
for l = 1:size(fq,2)-1
    if ~n_ex(l)
        fq_w_Err(:,2*(n-1)+1) = fq(:,l);
        fq_w_Err(:,2*n) = err_fq(:,l);
        q_new(n) = q(l);
        n = n+1;
    end
end

t = t_frame/frame_rate;
q = q_new;
fprintf('FINISHED! \n')
%%
if bleach_correction
    norm_fact = fq(:,end)./fq(1,end);
    for i = 1:size(fq_w_Err,2)
        fq_w_Err_new(:,i) = fq_w_Err(:,i)./norm_fact;
    end
    
    for i = 1:size(fq,2)
        fq_new(:,i) = fq(:,i)./norm_fact;
    end
    
else
    fq_w_Err_new = fq_w_Err;
    fq_new = fq;
end
%%
figure
    hold on
    for i = 1:5:size(fq_w_Err,1)/2
        errorbar(q,fq_w_Err(i,1:2:end),fq_w_Err(i,2:2:end));
    end
    set(gca,'XScale','log')
    set(gca,'YScale','log')
    xlabel('Wavevector Q [\mum^{-1}]')
    ylabel('Dynamic structure function \Delta(Q,t)')
    
figure
    hold on
    for i = 1:5:size(fq_w_Err,2)/2
        errorbar(t,fq_w_Err(:,2*(i-1)+1),fq_w_Err(:,2*i));
    end
    set(gca,'XScale','log')
    set(gca,'YScale','log')
    xlabel('Time [s]')
    ylabel('Dynamic structure function \Delta(Q,t)')
    
    %%
if SingleExpFit
    [SingleA,SingleB,SingleG] = DDM_SingleExpFit(fq_w_Err_new,t,SingleInitGuess,Singlelb,Singleub);
    
    figure
        errorbar(q,SingleA(:,1),SingleA(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Baseline A')
        
    figure
        errorbar(q,SingleB(:,1),SingleB(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Contrast B')
        
    figure
        errorbar(q,SingleG(:,1),SingleG(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Relaxation Rate \Gamma [s^{-1}]')
     
     t_fit = logspace(-3,3,100);
     fits_SingleExp = zeros(numel(t_fit),numel(q));
     
     for i = 1:numel(q)
         fits_SingleExp(:,i) = SingleA(i,1) + SingleB(i,1).*(1-exp(-SingleG(i,1).*t_fit));
     end
    
end

%%
if StretchExpFit
    [StretchA,StretchB,StretchG,StretchAlpha,StretchG_norm] = DDM_StretchExpFit(fq_w_Err_new,t,StretchInitGuess,Stretchlb,Stretchub,StretchDynamFit,StretchFixBaseline);
    
    figure
        errorbar(q,StretchA(:,1),StretchA(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Stretch Exp Baseline A')
        
    figure
        errorbar(q,StretchB(:,1),StretchB(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Stretch Exp Contrast B')
        
    figure
        errorbar(q,StretchG(:,1),StretchG(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Stretch Exp Relaxation Rate \Gamma [s^{-1}]')
        
    figure
        errorbar(q,StretchAlpha(:,1),StretchAlpha(:,2),'o')
        set(gca,'XScale','log')
        ylim([-0.2 2.2])
        xlabel('Wavevector [A]')
        ylabel('Stretch Exponent \alpha')
        
    figure
        errorbar(q,StretchG_norm(:,1),StretchG_norm(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Stretch Exp Norm Relaxation Rate <\Gamma> [s^{-1}]')
   
    t_fit = logspace(-3,3,100);
    fits_StretchExp = zeros(numel(t_fit),numel(q));
     for i = 1:numel(q)
         fits_StretchExp(:,i) = StretchA(i,1) + StretchB(i,1).*(1-exp(-(StretchG(i,1).*t_fit).^StretchAlpha(i,1)));
     end
    
end

%%
if DoubleExpFit
    [DE_A,DE_B,DE_f,DE_G1,DE_G2] = DDM_DoubleExpFit(fq_w_Err_new,t,DEInitGuess,DElb,DEub);
    
    figure
        errorbar(q,DE_A(:,1),DE_A(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Double Exp Baseline A')
        
    figure
        errorbar(q,DE_B(:,1),DE_B(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Double Exp Contrast B')
        
    figure
        errorbar(q,DE_f(:,1),DE_f(:,2),'o')
        set(gca,'XScale','log')
%         set(gca,'YScale','log')
        ylim([-0.2 1.2])
        xlabel('Wavevector [A]')
        ylabel('Double Exp Intensity Ratio f')
        
    figure
        errorbar(q,DE_G1(:,1),DE_G1(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Double Exponential Relaxation Rate \Gamma_1 [s^{-1}]')
        
    figure
        errorbar(q,DE_G2(:,1),DE_G2(:,2),'o')
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        xlabel('Wavevector [A]')
        ylabel('Double Exponential Relaxation Rate \Gamma_2 [s^{-1}]')
        
    t_fit = logspace(-3,3,100);
    fits_DoubleExp = zeros(numel(t_fit),numel(q));
     for i = 1:numel(q)
         fits_DoubleExp(:,i) = DE_A(i,1) + DE_B(i,1).*(1-((1-DE_f(i,1)).*exp(-(DE_G1(i,1).*t_fit))+DE_f(i,1).*exp(-(DE_G2(i,1).*t_fit))));
     end

end