function [ FFTav ] = FFTImDiff( varargin )
%FFTImDiff Calculates fourier transforms of image differences from avi
%files
%   This function can take either 2 or 3 variables. The first variable
%   should be the output of imfinfo on a TIFF. The second is the lag_time at
%   which we want to calculate the average fourier difference. The optional
%   third is a specified number of frames to average instead of varying
%   them
    movie = varargin{1};
    lag_time = varargin{2};
    FFTtot = 0;
    
    if nargin == 3
        num_average = varargin{3};
        if (num_average + lag_time) > numel(movie)
            fprintf('Error: trying to average too many frames. Setting number averaged to %d \n', movie.NumberOfFrames-lag_time)
            num_average = numel(movie) - lag_time;
        end
        for i = 1:num_average
            a = imread(movie(i).Filename,i);
            b = imread(movie(lag_time+i).Filename,lag_time+i);  %b is the image one delay_time away from a
            imdiff = b-a;                   %This is the image difference
            F = fft2(imdiff);               %Calculates the 2D fourier transfrom
            absFsq = abs(F.^2);  %Absolute value of the square of the fourier transform
            Fshift = fftshift(absFsq);      %Puts the "circle" at the center of the image
            FFTtot = FFTtot + Fshift;       %Adds current FFT to total FFT
%             fprintf('Averaging %d out of %d \n', i, num_average);
        end
    else
        for i = 1:(numel(movie) - lag_time)
            a = imread(movie(i).Filename,i);
            b = imread(movie(lag_time+i).Filename,lag_time+i);  %b is the image one delay_time away from a
            imdiff = b-a;                   %This is the image difference
            F = fft2(imdiff);               %Calculates the 2D fourier transfrom
            absFsq = abs(F.^2);  %Absolute value of the square of the fourier transform
            Fshift = fftshift(absFsq);      %Puts the "circle" at the center of the image
            FFTtot = FFTtot + Fshift;       %Adds current FFT to total FFT
%             fprintf('Averaging %d out of %d \n', i, movie.NumberOfFrames - lag_time);
        end
    end;
    FFTav = FFTtot/i;               %Averages the FFT over how many were added together
    
end

